import React from "react";
import ReactDOM from "react-dom"; 

const App = () => {
  const [genres, setGenres] = React.useState([]);
  const [filteredGenres, setFilteredGenres] = React.useState([]);
  const [filters, setFilters] = React.useState({
    era: "All", 
    origin: "All",
    context: "All",
    genre: "All",
  });
  const [filterOptions, setFilterOptions] = React.useState({
    era: [], 
    origin: [],
    context: [],
    genre: [],
  });

  // States for Apple query
  const [appleCountry, setAppleCountry] = React.useState("us");
  const [appleExplicit, setAppleExplicit] = React.useState(false);
  const [appleDataType, setAppleDataType] = React.useState("topsongs");
  const [currentPreview, setCurrentPreview] = React.useState(null);
  const [currentPlayingIndices, setCurrentPlayingIndices] = React.useState({
    genreIndex: null,
    songIndex: null,
  });
  const [activeGenreIndices, setActiveGenreIndices] = React.useState([]);
  const [allAppleResults, setAllAppleResults] = React.useState({});
  const [audioElement, setAudioElement] = React.useState(null);

  React.useEffect(() => {
    fetch("./assets/apple_genres clean.json")
      .then((response) => response.json())
      .then((data) => {
        console.log("Fetched genres:", data);
        setGenres(data);
        setFilteredGenres(data);

        // Dynamically generate filter options
        const eras = [...new Set(data.map((genre) => genre.era))]; 
        const origins = [...new Set(data.map((genre) => genre.origin))];
        const contexts = [
          ...new Set(data.map((genre) => genre.context || "Unknown")),
        ];
        const genres = [...new Set(data.map((genre) => genre.genre))];

        setFilterOptions({
          era: eras, 
          origin: origins,
          context: contexts,
          genre: genres,
        });

        const savedResults = localStorage.getItem("appleGenreResults");
        if (savedResults) { 
          const parsedResults = JSON.parse(savedResults);
          setAllAppleResults(parsedResults); 
        }
      });
  }, []);

  const handleFilterChange = (property, value) => {
    console.log(`Filter changed: ${property} = ${value}`);

    const newFilters = { ...filters };
    newFilters[property] = value;

    console.log("Updated filters:", newFilters);

    let newFilteredGenres = [...genres];

    // Apply filters one by one
    if (newFilters.era !== "All") {
      newFilteredGenres = newFilteredGenres.filter(
        (genre) => genre.era === newFilters.era
      );
      console.log(`Filtered by era: ${newFilters.era}`, newFilteredGenres);
    } 
    if (newFilters.origin !== "All") {
      newFilteredGenres = newFilteredGenres.filter(
        (genre) => genre.origin === newFilters.origin
      );
      console.log(
        `Filtered by origin: ${newFilters.origin}`,
        newFilteredGenres
      );
    }
    if (newFilters.context !== "All") {
      newFilteredGenres = newFilteredGenres.filter(
        (genre) => genre.context === newFilters.context
      );
    }
    if (newFilters.genre !== "All") {
      newFilteredGenres = newFilteredGenres.filter(
        (genre) => genre.genre === newFilters.genre
      );
    }

    console.log("Final filtered genres:", newFilteredGenres);

    setFilters(newFilters);
    setFilteredGenres(newFilteredGenres);
  };

  // Helper function to generate a consistent storage key for a genre query
  const getGenreQueryKey = (genre, country, dataType, explicit) => {
    const genreCode =
      genre.subgenre && genre.subgenre.length > 0
        ? genre.subgenre[0]
        : "unknown";
    return `${genre.name}_${genreCode}_${country}_${dataType}_${explicit}`;
  };

  // Updated handleGenreAppleQuery function
  const handleGenreAppleQuery = (genre, index) => {
    const genreCode =
      genre.subgenre && genre.subgenre.length > 0 ? genre.subgenre[0] : null;
    if (!genreCode) return;

    const queryKey = getGenreQueryKey(
      genre,
      appleCountry,
      appleDataType,
      appleExplicit
    );
    const isRefresh = activeGenreIndices.includes(index);

    // Always set active index, even for cached results
    if (!isRefresh) {
      setActiveGenreIndices((prev) => [...prev, index]);
    }

    // Skip cache checks if this is a refresh request
    if (!isRefresh) {
      // Check if we already have the results in state
      if (allAppleResults[queryKey]) {
        console.log("Loading cached results from state for:", genre.name);
        return;
      }

      // Check localStorage before making the API call
      const savedData = localStorage.getItem("appleGenreResults");
      if (savedData) {
        const parsedData = JSON.parse(savedData);
        if (parsedData[queryKey]) {
          console.log(
            "Loading cached results from localStorage for:",
            genre.name,
            parsedData[queryKey]
          );
          setAllAppleResults((prevResults) => ({
            ...prevResults,
            [queryKey]: parsedData[queryKey],
          }));
          return;
        }
      }
    }

    const itunesUrl = `https://itunes.apple.com/${appleCountry}/rss/${appleDataType}/limit=100/genre=${genreCode}/explicit=${
      appleExplicit ? "true" : "false"
    }/json`;

    console.log(
      isRefresh ? "Refreshing data from API" : "Fetching new data from API"
    );
    fetch(itunesUrl)
      .then((response) => response.json())
      .then((data) => {
        let entries = data?.feed?.entry || []; 
        const entryList = entries.map((entry) => { 
          let returnThis = {
            title: entry["im:name"].label,
            artist: entry["im:artist"].label,
            previewLink: entry.link[1]?.attributes?.href || "",
          }; 
          return returnThis;
        });

        console.log("Fetched data from API:", entryList);

        const newAllResults = { ...allAppleResults };
        newAllResults[queryKey] = entryList;
        setAllAppleResults(newAllResults);
        localStorage.setItem('appleGenreResults', JSON.stringify(newAllResults));
      })
      .catch((error) => console.error("Error loading data: ", error));
  };

  // Updated audio preview handler
  const playPreview = (previewLink, genreIndex, songIndex) => {
    // Stop any existing audio
    if (audioElement) {
      audioElement.pause();
      audioElement.src = "";
    }

    if (
      currentPlayingIndices.genreIndex === genreIndex &&
      currentPlayingIndices.songIndex === songIndex
    ) {
      setCurrentPreview(null);
      setCurrentPlayingIndices({ genreIndex: null, songIndex: null });
      setAudioElement(null);
    } else {
      setCurrentPreview(previewLink);
      setCurrentPlayingIndices({ genreIndex, songIndex });

      // Create and configure new audio element
      const newAudio = new Audio(previewLink);
      newAudio.addEventListener("ended", () => {
        setCurrentPreview(null);
        setCurrentPlayingIndices({ genreIndex: null, songIndex: null });
        setAudioElement(null);
      });
      newAudio.play();
      setAudioElement(newAudio);
    }
  };

  const handleDetailsToggle = (e, genre, index) => {
    if (e.target.open) {
      const queryKey = getGenreQueryKey(
        genre,
        appleCountry,
        appleDataType,
        appleExplicit
      );
      
      // Check if we have cached results
      if (allAppleResults[queryKey]) {
        setActiveGenreIndices((prev) => [...prev, index]);
      } else {
        // If no cached results, fetch them automatically
        handleGenreAppleQuery(genre, index);
      }
    }
  };

  const clearLocalStorage = () => {
    localStorage.removeItem('appleGenreResults');
    setAllAppleResults({});
    setActiveGenreIndices([]);
  };

  return (
    <div className="container py-4">
      <div className="bg-light p-4 mb-4 rounded">
        <h2 className="mt-0">iTunes Music Explorer</h2>
        <p className="mb-0">
          This tool allows you to explore music from the iTunes catalog using Apple's RSS feeds. 
          You can browse different genres, filter by various criteria, and listen to song previews. 
          The data is fetched from Apple's iTunes API and cached locally for better performance.
        </p>
      </div>  
      <div className="mb-4">
        <h2 className="h4 mb-3">Filter the Apple Feed</h2>
        <div className="mb-3">
          <label htmlFor="countryDropdown" className="form-label">Country:</label>
          <input
            type="text"
            id="countryDropdown"
            className="form-control"
            value={appleCountry}
            onChange={(e) => setAppleCountry(e.target.value)}
            placeholder="eg. us"
          />
        </div>
        <div className="mb-3 form-check">
          <input
            type="checkbox"
            id="explicitCheckbox"
            className="form-check-input"
            checked={appleExplicit}
            onChange={(e) => setAppleExplicit(e.target.checked)}
          />
          <label htmlFor="explicitCheckbox" className="form-check-label">Explicit</label>
        </div>
        <div className="mb-3">
          <span className="d-block mb-2">Data Type:</span>
          <div className="btn-group" role="group">
            <label className="btn btn-outline-primary">
              <input
                type="radio"
                name="dataType"
                value="topsongs"
                className="btn-check"
                checked={appleDataType === "topsongs"}
                onChange={(e) => setAppleDataType(e.target.value)}
              />
              Top Songs
            </label>
            <label className="btn btn-outline-primary">
              <input
                type="radio"
                name="dataType"
                value="topalbums"
                className="btn-check"
                checked={appleDataType === "topalbums"}
                onChange={(e) => setAppleDataType(e.target.value)}
              />
              Top Albums
            </label>
          </div>
        </div>
        <div>
          <button 
            onClick={clearLocalStorage}
            style={{
              marginTop: '1rem',
              backgroundColor: '#ff4444',
              color: 'white',
              padding: '0.5rem',
              border: 'none',
              borderRadius: '4px',
              cursor: 'pointer'
            }}
          >
            Clear Cached Results
          </button>
        </div>
      </div>

      <div>
        <h3>Filter by:</h3>
        <div style={{ marginBottom: '1rem' }}>
          <div className="mb-2">Era:</div>
          <div className="btn-group flex-wrap" role="group" aria-label="Era filter">
            <button 
              type="button" 
              className={`btn ${filters.era === "All" ? "btn-primary" : "btn-outline-primary"}`}
              onClick={() => handleFilterChange("era", "All")}
            >
              All
            </button>
            {filterOptions.era.map((era) => (
              <button 
                key={era}
                type="button" 
                className={`btn ${filters.era === era ? "btn-primary" : "btn-outline-primary"}`}
                onClick={() => handleFilterChange("era", era)}
              >
                {era}
              </button>
            ))}
          </div>
        </div> 

        <div style={{ marginBottom: '1rem' }}>
          <div className="mb-2">Origin:</div>
          <div className="btn-group flex-wrap" role="group" aria-label="Origin filter">
            <button 
              type="button" 
              className={`btn ${filters.origin === "All" ? "btn-primary" : "btn-outline-primary"}`}
              onClick={() => handleFilterChange("origin", "All")}
            >
              All
            </button>
            {filterOptions.origin.map((origin) => (
              <button 
                key={origin}
                type="button" 
                className={`btn ${filters.origin === origin ? "btn-primary" : "btn-outline-primary"}`}
                onClick={() => handleFilterChange("origin", origin)}
              >
                {origin}
              </button>
            ))}
          </div>
        </div>

        <div style={{ marginBottom: '1rem' }}>
          <div className="mb-2">Context:</div>
          <div className="btn-group flex-wrap" role="group" aria-label="Context filter">
            <button 
              type="button" 
              className={`btn ${filters.context === "All" ? "btn-primary" : "btn-outline-primary"}`}
              onClick={() => handleFilterChange("context", "All")}
            >
              All
            </button>
            {filterOptions.context.map((context) => (
              <button 
                key={context}
                type="button" 
                className={`btn ${filters.context === context ? "btn-primary" : "btn-outline-primary"}`}
                onClick={() => handleFilterChange("context", context)}
              >
                {context}
              </button>
            ))}
          </div>
        </div>

        <div style={{ marginBottom: '1rem' }}>
          <div className="mb-2">Genre:</div>
          <div className="btn-group flex-wrap" role="group" aria-label="Genre filter">
            <button 
              type="button" 
              className={`btn ${filters.genre === "All" ? "btn-primary" : "btn-outline-primary"}`}
              onClick={() => handleFilterChange("genre", "All")}
            >
              All
            </button>
            {filterOptions.genre.map((genre) => (
              <button 
                key={genre}
                type="button" 
                className={`btn ${filters.genre === genre ? "btn-primary" : "btn-outline-primary"}`}
                onClick={() => handleFilterChange("genre", genre)}
              >
                {genre}
              </button>
            ))}
          </div>
        </div>
      </div>

      {/* Updated genres list with inline Apple results display */}
      <div>
        <h3>Genres:</h3>
        {filteredGenres.map((genre, index) => {
          const queryKey = getGenreQueryKey(
            genre,
            appleCountry,
            appleDataType,
            appleExplicit
          );
          const genreResults = allAppleResults[queryKey] || [];

          return (
            <details 
              key={genre.name} 
              onToggle={(e) => handleDetailsToggle(e, genre, index)}
            >
              <summary>{genre.name}</summary>
              <button onClick={() => handleGenreAppleQuery(genre, index)}>
                {activeGenreIndices.includes(index) ? "Refresh" : "Submit"}
              </button>
              {activeGenreIndices.includes(index) && (
                <div>
                  <table
                    id="itunesDataList"
                    border="1"
                    style={{ marginTop: "1rem" }}
                  >
                    <thead>
                      <tr>
                        <th>Title</th>
                        <th>Artist</th>
                        <th>Preview</th>
                        <th>YouTube Search</th>
                      </tr>
                    </thead>
                    <tbody>
                      {genreResults.map((entry, i) => (
                        <tr key={i}>
                          <td>{entry.title}</td>
                          <td>{entry.artist}</td>
                          <td>
                            <button
                              onClick={() =>
                                playPreview(entry.previewLink, index, i)
                              }
                              style={{
                                backgroundColor:
                                  currentPlayingIndices.genreIndex === index &&
                                  currentPlayingIndices.songIndex === i
                                    ? "red"
                                    : "green",
                              }}
                            >
                              {currentPlayingIndices.genreIndex === index &&
                              currentPlayingIndices.songIndex === i
                                ? "Stop"
                                : "Preview"}
                            </button>
                          </td>
                          <td>
                            <a
                              href={`https://www.youtube.com/results?search_query=${encodeURIComponent(
                                entry.artist
                              )}+ - +${encodeURIComponent(entry.title)}`}
                              target="_blank"
                              rel="noreferrer"
                            >
                              Search YouTube
                            </a>
                          </td>
                        </tr>
                      ))}
                    </tbody>
                  </table>
                  {currentPreview &&
                    currentPlayingIndices.genreIndex === index && (
                      <div style={{ marginTop: "1rem" }}>
                        {/* Remove the audio tag since we're handling it through the Audio API */}
                      </div>
                    )}
                </div>
              )}
            </details>
          );
        })}
      </div>
    </div>
  );
};

ReactDOM.render(<App />, document.getElementById("react-root"));
